// ----------------------------------------------------------------------------
// This reference design and source code is being provided on an "as-is" basis 
// and as an accommodation, and therefore all warranties, representations or 
// guarantees of any kind (whether express, implied or statutory) including, 
// without limitation, warranties of merchantability, non-infringement, or  
// fitness for a particular purpose, are specifically disclaimed.
//
// This source code may only be used in an Altera programmable logic device
// and may not be distributed without permission from Macnica Americas, Inc.  
// It is provided free of royalties or fees of any kind.
// ----------------------------------------------------------------------------
//
// Description  | Simple freqency detection
//              | 
//              | PLL generates three fixed-freq examples:
//              | 	0.6 MHz
//              | 	14.3 MHz
//              | 	25.0 MHz
//              | 
//              | I2C register controls which sample freq is fed to freq detect circuit
//              | Freq detect outputs 8 bit result of 0.1's of MHz (~0.1 MHz to 25.5 MHz +/-0.1 MHz)
//              | This 8 bit result is displayed to LEDs and written to the "auto" register
//              | which is auto-montiored by phone app for display
//              | 
//              | I2C Register Map:
//              | Base = 0x30 (Wr base: 0x60, Rd base: 0x61)
//              |		00: AutoReg
//              | 		8 bit result in 0.1's of MHz for the measured input
//              | 	01: sample mux control
//              | 		[7:3] = unused
//              | 		[2:0] = sample signal
//					 |				000 = input frequency
//              | 			001 = sample1 (0.6 MHz)
//              | 			010 = sample2 (14.3 MHz)
//              | 			011 = sample3 (25.0 MHz)
//              | 
//					 | External frequency can be inserted on J4 pin 3. 
//					 | Frequency selected can be read back on J4 pin 15. 
//              | 
// Formatting   | Tabs set at 3 spaces.  Port names 'UPPER' case.
//              | Internal wires and registers are 'lower' case.
//              | Module names 'lower' case to match lower case filename.v
//              | Parameters are first character 'Upper' case.
//              | Active low signals are identified with '_n' or '_N'
//              | appended to the wire, register, or port name.
//              | 
//              | _r suffix on a register name indicates a simple
//              | re-registering of another wire or register.
//              | 
//              | _rr suffix on a register name indicates a second
//              | re-registering of a register.
//              | 
//              | _metastab indicates clock domain crossing
//
// ----------------------------------------------------------------------------

`timescale 1ns/1ps

//--------------------------
//	Declaration and ports
//-----------
module top (
	
	//global
	input					IN_CLOCK,
	
	//i2c
	input					IN_SCL,
	inout					INOUT_SDA,
	
	//gpio
	output	[7:0]		OUT_LED,
	input		[2:0]		IN_SWITCH,
	input		[1:0]		IN_BUTTON_N,
	
	// frequency input and output
	input 				IN_FREQ,
	output				OUT_FREQ,
	output				OUT_MEASURE1,
	output				OUT_MEASURE2,
	output				OUT_MEASURE3	
);


//--------------------------
//	Regs and Wires
//-----------
	reg		[7:0]		autoreg;
	reg					reset_n_r;
	reg					reset_n_rr;
	reg					clk2measure;	
	wire					clock50mhz;
	wire					clock100mhz;
	wire					locked;
	wire					example1;
	wire					example2;
	wire					example3;
	wire		[7:0]		clkselect;
	wire		[7:0]		result;


//--------------------------
//	The I2C Slave
//-----------
i2cSlave i2c_inst(
	.clk(clock50mhz),
	.rst(!reset_n_rr),
	.sda(INOUT_SDA),
	.scl(IN_SCL),
	.reg0_auto_in(autoreg[7:0]),
	.reg1_led_out(clkselect[7:0]), //use what was for LED control as example select
	.reg2_switch_in({5'b0,IN_SWITCH[2:0]}),
	.reg3_button_in({6'b0,IN_BUTTON_N[1:0]})
);


//--------------------------
//	PLL
//-----------
pll pll_inst (
	.inclk0 ( IN_CLOCK ),
	.c0 ( clock50mhz ),
	.c1 ( clock100mhz ),
	.c2 ( example1 ),
	.c3 ( example2 ),
	.c4 ( example3 ),
	.locked ( locked )
	);

//--------------------------
//	Reset - Synchronous deassert after PLL locked
//-----------
//Synchronize Reset
always @(posedge IN_CLOCK)
	begin
	if (!locked)
		begin
		reset_n_r <= 1'b0;
		reset_n_rr <= 1'b0;
		end
	else
		begin
		reset_n_r <= 1'b1;
		reset_n_rr <= reset_n_r;
		end
	end


//--------------------------
//	Encode the auto register
//-----------
always @(posedge clock50mhz)
	begin
	if (!reset_n_rr)
		begin
		autoreg[7:0] <= 8'h00;
		end
	else
		begin
		autoreg[7:0] <= result[7:0];
		end
	end


//--------------------------
//	Mux for clk2measure
//-----------
always @(clkselect[2:0] or IN_FREQ or example1 or example2 or example3)
	begin
	case (clkselect[2:0])
		(4'h0): clk2measure = IN_FREQ;  // Input frequency from J4 pin 3
		(4'h1): clk2measure = example1;
		(4'h2): clk2measure = example2;
		(4'h3): clk2measure = example3;
		default: clk2measure = 1'b0;
	endcase
	end

assign OUT_FREQ = clk2measure; // Output frequency to J4 pin 15

// Provide outputs that can be fed back into the IN_FREQ pin
assign OUT_MEASURE1 = example1; // Output example freq 1 to J4 pin 4
assign OUT_MEASURE2 = example2; // Output example freq 2 to J4 pin 5
assign OUT_MEASURE3 = example3; // Output example freq 3 to J4 pin 6
	
//--------------------------
//	The Frequency Detect
//-----------
freqdetect fd_inst (
	//global
	.IN_CLOCK100MHZ(clock100mhz),
	.IN_RESET_N(reset_n_rr),
	//clk to monitor
	.IN_CLK2MEASURE(clk2measure),
	//result
	.OUT_RESULT(result[7:0])
);
assign OUT_LED = ~result;

endmodule